"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Component_1 = __importDefault(require("../share/Component"));
const tool_highlight_1 = require("./overlay/tool_highlight");
const util_1 = require("../share/util");
const ResizeSensor_1 = __importDefault(require("licia/ResizeSensor"));
const throttle_1 = __importDefault(require("licia/throttle"));
const lowerCase_1 = __importDefault(require("licia/lowerCase"));
const each_1 = __importDefault(require("licia/each"));
const Color_1 = __importDefault(require("licia/Color"));
const hex_1 = __importDefault(require("licia/hex"));
const upperCase_1 = __importDefault(require("licia/upperCase"));
const extend_1 = __importDefault(require("licia/extend"));
const camelCase_1 = __importDefault(require("licia/camelCase"));
const contain_1 = __importDefault(require("licia/contain"));
const toNum_1 = __importDefault(require("licia/toNum"));
const elementRoles_1 = __importDefault(require("./elementRoles"));
class DomHighlighter extends Component_1.default {
    constructor(container, { showRulers = false, showExtensionLines = false, showInfo = true, showStyles = true, showAccessibilityInfo = true, colorFormat = 'hex', contentColor = 'rgba(111, 168, 220, .66)', paddingColor = 'rgba(147, 196, 125, .55)', borderColor = 'rgba(255, 229, 153, .66)', marginColor = 'rgba(246, 178, 107, .66)', monitorResize = true, } = {}) {
        super(container, { compName: 'dom-highlighter' });
        this.overlay = new tool_highlight_1.HighlightOverlay(window);
        this.reset = () => {
            const viewportWidth = document.documentElement.clientWidth;
            const viewportHeight = document.documentElement.clientHeight;
            this.overlay.reset({
                viewportSize: {
                    width: viewportWidth,
                    height: viewportHeight,
                },
                deviceScaleFactor: 1,
                pageScaleFactor: 1,
                pageZoomFactor: 1,
                emulationScaleFactor: 1,
                scrollX: window.scrollX,
                scrollY: window.scrollY,
            });
        };
        this.options = {
            showRulers,
            showExtensionLines,
            showInfo,
            showStyles,
            showAccessibilityInfo,
            colorFormat,
            contentColor,
            paddingColor,
            borderColor,
            marginColor,
            monitorResize,
        };
        this.overlay.setContainer(container);
        this.overlay.setPlatform('mac');
        this.redraw = throttle_1.default(() => {
            this.reset();
            this.draw();
        }, 16);
        this.redraw();
        this.bindEvent();
    }
    highlight(target) {
        this.target = target;
        if (target instanceof HTMLElement && this.options.monitorResize) {
            if (this.resizeSensor) {
                this.resizeSensor.destroy();
            }
            this.resizeSensor = new ResizeSensor_1.default(target);
            this.resizeSensor.addListener(this.redraw);
        }
        this.redraw();
    }
    hide() {
        this.target = null;
        this.redraw();
    }
    setOption(name, val) {
        const options = this.options;
        const oldVal = options[name];
        options[name] = val;
        this.emit('optionChange', val, oldVal);
        this.redraw();
    }
    intercept(interceptor) {
        this.interceptor = interceptor;
    }
    destroy() {
        window.removeEventListener('resize', this.redraw);
        window.removeEventListener('scroll', this.redraw);
        if (this.resizeSensor) {
            this.resizeSensor.destroy();
        }
        super.destroy();
    }
    draw() {
        const { target } = this;
        if (!target) {
            return;
        }
        if (target instanceof Text) {
            this.drawText(target);
        }
        else {
            this.drawElement(target);
        }
    }
    drawText(target) {
        const range = document.createRange();
        range.selectNode(target);
        const { left, top, width, height } = range.getBoundingClientRect();
        range.detach();
        const highlight = {
            paths: [
                {
                    path: this.rectToPath({
                        left,
                        top,
                        width,
                        height,
                    }),
                    fillColor: this.options.contentColor,
                    name: 'content',
                },
            ],
            showExtensionLines: this.options.showExtensionLines,
            showRulers: this.options.showRulers,
        };
        if (this.options.showInfo) {
            highlight.elementInfo = {
                tagName: '#text',
                nodeWidth: width,
                nodeHeight: height,
            };
        }
        this.overlay.drawHighlight(highlight);
    }
    drawElement(target) {
        let highlight = {
            paths: this.getPaths(target),
            showExtensionLines: this.options.showExtensionLines,
            showRulers: this.options.showRulers,
            colorFormat: this.options.colorFormat,
        };
        if (this.options.showInfo) {
            highlight.elementInfo = this.getElementInfo(target);
        }
        if (this.interceptor) {
            const result = this.interceptor(highlight);
            if (result) {
                highlight = result;
            }
        }
        this.overlay.drawHighlight(highlight);
    }
    getPaths(target) {
        const computedStyle = window.getComputedStyle(target);
        const { left, top, width, height } = target.getBoundingClientRect();
        const getNumStyle = (name) => {
            return util_1.pxToNum(computedStyle.getPropertyValue(name));
        };
        const ml = getNumStyle('margin-left');
        const mr = getNumStyle('margin-right');
        const mt = getNumStyle('margin-top');
        const mb = getNumStyle('margin-bottom');
        const bl = getNumStyle('border-left-width');
        const br = getNumStyle('border-right-width');
        const bt = getNumStyle('border-top-width');
        const bb = getNumStyle('border-bottom-width');
        const pl = getNumStyle('padding-left');
        const pr = getNumStyle('padding-right');
        const pt = getNumStyle('padding-top');
        const pb = getNumStyle('padding-bottom');
        const contentPath = {
            path: this.rectToPath({
                left: left + bl + pl,
                top: top + bt + pt,
                width: width - bl - pl - br - pr,
                height: height - bt - pt - bb - pb,
            }),
            fillColor: this.options.contentColor,
            name: 'content',
        };
        const paddingPath = {
            path: this.rectToPath({
                left: left + bl,
                top: top + bt,
                width: width - bl - br,
                height: height - bt - bb,
            }),
            fillColor: this.options.paddingColor,
            name: 'padding',
        };
        const borderPath = {
            path: this.rectToPath({
                left,
                top,
                width,
                height,
            }),
            fillColor: this.options.borderColor,
            name: 'border',
        };
        const marginPath = {
            path: this.rectToPath({
                left: left - ml,
                top: top - mt,
                width: width + ml + mr,
                height: height + mt + mb,
            }),
            fillColor: this.options.marginColor,
            name: 'margin',
        };
        return [contentPath, paddingPath, borderPath, marginPath];
    }
    getElementInfo(target) {
        const { width, height } = target.getBoundingClientRect();
        const className = target.className
            .split(/\s+/)
            .map((c) => '.' + c)
            .join('');
        const elementInfo = {
            tagName: lowerCase_1.default(target.tagName),
            className,
            idValue: target.id,
            nodeWidth: width,
            nodeHeight: height,
        };
        if (this.options.showStyles) {
            elementInfo.style = this.getStyles(target);
        }
        if (this.options.showAccessibilityInfo) {
            extend_1.default(elementInfo, this.getAccessibilityInfo(target));
        }
        return elementInfo;
    }
    // third_party/blink/renderer/core/inspector/inspector_highlight.cc
    getStyles(target) {
        const computedStyle = window.getComputedStyle(target);
        let hasTextChildren = false;
        const childNodes = target.childNodes;
        for (let i = 0, len = childNodes.length; i < len; i++) {
            if (childNodes[i].nodeType === 3) {
                hasTextChildren = true;
            }
        }
        const properties = [];
        if (hasTextChildren) {
            properties.push('color', 'font-family', 'font-size', 'line-height');
        }
        properties.push('padding', 'margin', 'background-color');
        return propertiesToValues(computedStyle, properties);
    }
    getAccessibilityInfo(target) {
        const computedStyle = window.getComputedStyle(target);
        return {
            showAccessibilityInfo: true,
            contrast: {
                contrastAlgorithm: 'aa',
                textOpacity: 0.1,
                ...propertiesToValues(computedStyle, ['font-size', 'font-weight', 'background-color', 'text-opacity'], true),
            },
            isKeyboardFocusable: this.isFocusable(target),
            ...this.getAccessibleNameAndRole(target),
        };
    }
    isFocusable(target) {
        const tagName = lowerCase_1.default(target.tagName);
        if (contain_1.default(['a', 'button', 'input', 'textarea', 'select', 'details'], tagName)) {
            return true;
        }
        const tabIdx = target.getAttribute('tabindex');
        if (tabIdx && toNum_1.default(tabIdx) > -1) {
            return true;
        }
        return false;
    }
    getAccessibleNameAndRole(target) {
        const name = target.getAttribute('labelledby') || target.getAttribute('aria-label');
        let role = target.getAttribute('role');
        const tagName = lowerCase_1.default(target.tagName);
        elementRoles_1.default.forEach((value) => {
            if (role) {
                return;
            }
            const name = value[0];
            const attributes = value[2];
            if (name !== tagName) {
                return;
            }
            if (attributes) {
                for (const attribute of attributes) {
                    if (target.getAttribute(attribute[0]) !== attribute[1]) {
                        return;
                    }
                }
            }
            role = value[1];
        });
        return {
            accessibleName: name || target.getAttribute('title') || '',
            accessibleRole: role || 'generic',
        };
    }
    bindEvent() {
        window.addEventListener('resize', this.redraw);
        window.addEventListener('scroll', this.redraw);
    }
    rectToPath({ left, top, width, height }) {
        const path = [];
        path.push('M', left, top);
        path.push('L', left + width, top);
        path.push('L', left + width, top + height);
        path.push('L', left, top + height);
        path.push('Z');
        return path;
    }
}
exports.default = DomHighlighter;
module.exports = DomHighlighter;
module.exports.default = DomHighlighter;
const regRgb = /^rgb\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3})\)$/;
const regRgba = /^rgba\((\d{1,3}),\s*(\d{1,3}),\s*(\d{1,3}),\s*(\d*(?:\.\d+)?)\)$/;
function isColor(color) {
    return regRgb.test(color) || regRgba.test(color);
}
function rgbToHex(rgb) {
    const color = Color_1.default.parse(rgb);
    const opacity = color.val[3] || 1;
    color.val = color.val.slice(0, 3);
    color.val.push(Math.round(255 * opacity));
    return '#' + upperCase_1.default(hex_1.default.encode(color.val));
}
function propertiesToValues(computedStyle, properties, useCamelCase = false) {
    const ret = {};
    each_1.default(properties, (property) => {
        let value = computedStyle[property === 'text-opacity' ? 'color' : property];
        if (!value) {
            return;
        }
        if (isColor(value)) {
            value = rgbToHex(value);
            if (property === 'text-opacity') {
                value = value.slice(7);
                value = hex_1.default.decode(value)[0] / 255;
            }
        }
        if (useCamelCase) {
            property = camelCase_1.default(property);
        }
        ret[property] = value;
    });
    return ret;
}
